import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes, TicksPerSecond, GameMode } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

function removeSingleItem(item, player) {
  const equippable = player.getComponent("equippable");
  const mainHandItem = equippable.getEquipment(EquipmentSlot.Mainhand);

  if (mainHandItem && mainHandItem.typeId === item.typeId) {
    if ((item.amount == 1)) {
      equippable.setEquipment(
        EquipmentSlot.Mainhand,
        new ItemStack("minecraft:air")
      );
    } else {
      const newItemStack = new ItemStack(item.typeId, mainHandItem.amount - 1);
      equippable.setEquipment(EquipmentSlot.Mainhand, newItemStack);
    }
  }
}

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_admin:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_admin:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});

function roundCoordinates(value) {
  return Math.round(value * 100) / 100;
}


system.runInterval(() => {
  // Iterate over all players in the world
  world.getPlayers().forEach(player => {
    // Check if the player is in Spectator mode
    if (player.getGameMode() === GameMode.spectator) {
      // Get the player's view direction
      const viewDirection = player.getViewDirection();
      player.onScreenDisplay.setActionBar({text: "§aLook directly up to exit spectator."});
      
      // Check if the player is looking straight up
      const isLookingStraightUp = viewDirection.y > 0.99; // Adjust threshold if necessary

      if (isLookingStraightUp) {
        // Retrieve the player's previous gamemode from the tag
        const gamemodeTag = player.getTags().find(tag => tag.startsWith("sg_admin:spectate:"));
        if (gamemodeTag) {
          const previousGamemode = gamemodeTag.split(":")[2];
          
          // Revert the player's gamemode to their previous gamemode
          player.setGameMode(GameMode[previousGamemode.toLowerCase()]);
          
          // Remove the gamemode tag
          player.removeTag(gamemodeTag);
        }
        
        // Retrieve the player's old location from the tag
        const positionTag = player.getTags().find(tag => tag.startsWith("sg_admin:spectate_position:"));
        if (positionTag) {
          const [xStr, yStr, zStr] = positionTag.split(":")[2].split(",");

          world.sendMessage(`§e${player.name} joined the game`);
          // Teleport the player back to their old location
        const x = parseFloat(xStr);
        const y = parseFloat(yStr);
        const z = parseFloat(zStr);

        // Ensure the coordinates are valid numbers before teleporting
        if (!isNaN(x) && !isNaN(y) && !isNaN(z)) {
          const adjustedX = x + 0.5;
          const adjustedZ = z + 0.5;

          // Teleport the player to the calculated coordinates
          player.teleport({ x: adjustedX, y: y, z: adjustedZ });

          // Iterate over all tags of the player
          player.getTags().forEach((tag) => {
            // Check if the tag starts with "sg_admin:spectate_position:"
            if (tag.startsWith("sg_admin:spectate_position:")) {
              // Remove the tag
              player.removeTag(tag);
            }
          });
        }

        // Notify the player
        player.onScreenDisplay.setActionBar({text: "§aYou have returned to your previous gamemode and location."});
      }
    }
  }
}, 5); // Run the function every 5 ticks (adjust if necessary)
})





const kickPlayerForm = (adminPlayer) => {
  const players = world.getPlayers();
  const form = new ModalFormData()
    .title("Kick Player")
    .dropdown("Select Player", players.map(p => p.name))
    .textField("Reason", "Enter reason here", "Rule violation")
    .toggle("Broadcast Removal?", true);

  form.show(adminPlayer).then(res => {
    if (res.canceled) return;

    const [selectedPlayerIndex, reason, broadcast] = res.formValues;
    const playerName = players[selectedPlayerIndex].name;
    const isAdmin = world.scoreboard.getObjective(`sg_admin:admin_player:${playerName}`);
    const isOwner = players[selectedPlayerIndex].hasTag("sg_admin:owner");

    // Check if the reason is empty
    if (reason.trim() === "") {
      adminPlayer.sendMessage("§cReason cannot be blank. Kick operation aborted.");
      return;
    }

    if (isAdmin) {
      adminPlayer.sendMessage(
        `§c${playerName} is an admin and cannot be kicked.`
      );
    } else if (isOwner) {
      adminPlayer.sendMessage(
        `§c${playerName} is the owner and cannot be banned.`
      );
    } else {
      adminPlayer.runCommand(`kick ${playerName} §c${reason}`);

      if (broadcast) {
        world.sendMessage(
          `§e${playerName} was kicked from the world for: §c${reason}`
        );
      }

      adminPlayer.sendMessage(
        `§aYou have kicked ${playerName} for: §c${reason}`
      );
    }
  });
};



const banPlayerForm = (adminPlayer) => {
  const players = world.getPlayers();
  const form = new ModalFormData()
    .title("Ban Player")
    .dropdown("Select Player", players.map(p => p.name))
    .textField("Or Input Player's Name Here", "Enter player name", "")
    .textField("Reason", "Enter reason here", "")
    .toggle("Broadcast Ban?", true);

  form.show(adminPlayer).then(res => {
    if (res.canceled) return;

    const [selectedPlayerIndex, inputPlayerName, reason, broadcast] = res.formValues;
    const playerName = inputPlayerName.trim() !== "" ? inputPlayerName : players[selectedPlayerIndex].name;
    const isAdmin = world.scoreboard.getObjective(`sg_admin:admin_player:${playerName}`);
    const isOwner = players[selectedPlayerIndex].hasTag("sg_admin:owner");

    // Check if the reason is empty
    if (reason.trim() === "") {
      adminPlayer.sendMessage("§cReason cannot be blank. Ban operation aborted.");
      return;
    }

    if (isAdmin) {
      adminPlayer.sendMessage(`§c${playerName} is an admin and cannot be banned.`);
    } else if (isOwner) {
      adminPlayer.sendMessage(`§c${playerName} is the owner and cannot be banned.`);
    } else {
      const banObjectiveName = `sg_admin:banned_player:${playerName}`;
      world.scoreboard.addObjective(banObjectiveName, reason);
      adminPlayer.runCommand(
        `kick ${playerName} §cYou are now banned. §fReason: §c${reason}`
      );

      if (broadcast) {
        world.sendMessage(
          `§e${playerName} was banned from the server for: §c${reason}`
        );
      }

      adminPlayer.sendMessage(
        `§aYou have banned ${playerName} for: §c${reason}`
      );
    }
  });
};



const viewBannedPlayersForm = (adminPlayer) => {
  const objectives = world.scoreboard.getObjectives();
  const bannedPlayers = objectives.filter(obj => obj.id.startsWith("sg_admin:banned_player:"));

  if (bannedPlayers.length === 0) {
    const form = new ActionFormData()
      .title("Banned Players")
      .body("§7No players are currently banned.")
      .button("§oGo Back");

    form.show(adminPlayer).then(() => showMainPage(adminPlayer));
    return;
  }

  const form = new ActionFormData().title("Banned Players");
  
  bannedPlayers.forEach(ban => {
    const playerName = ban.id.split(":")[2];
    const reason = ban.displayName || "No reason provided";
    form.button(`${playerName} - ${reason}`, "textures/sg/admin/icons/bplayer.png");
  });
  form.button("§oGo Back");

  form.show(adminPlayer).then(res => {
    if (res.canceled || res.selection === undefined) return;

    if (res.selection === bannedPlayers.length) {
      showMainPage(adminPlayer); // Go back to the main page
    } else if (res.selection === bannedPlayers.length + 1) {
      viewAdminPlayersForm(adminPlayer); // View Admins
    } else {
      const selectedBan = bannedPlayers[res.selection];
      const playerName = selectedBan.id.split(":")[2];
      const reason = selectedBan.displayName || "No reason provided";

      viewUnbanForm(adminPlayer, playerName, reason);
    }
  });
};

const viewUnbanForm = (adminPlayer, playerName, reason) => {
  const form = new ActionFormData()
    .title(`Ban Details - ${playerName}`)
    .body(`§7Player: §f${playerName}\n§7Reason: §f${reason}\n\n§7What would you like to do?`)
    .button("Unban", "textures/sg/admin/icons/unban.png")
    .button("Go Back");

  form.show(adminPlayer).then(res => {
    if (res.canceled || res.selection === undefined) return;

    if (res.selection === 0) {
      // Unban the player
      const banObjectiveName = `sg_admin:banned_player:${playerName}`;
      const objective = world.scoreboard.getObjective(banObjectiveName);

      if (objective) {
        world.scoreboard.removeObjective(banObjectiveName);
        adminPlayer.sendMessage(`§aYou have unbanned ${playerName}.`);
      } else {
        adminPlayer.sendMessage(`§cFailed to unban. Player not found in ban list.`);
      }
    }

    viewBannedPlayersForm(adminPlayer); // Go back to banned players list
  });
};

const viewAdminPlayersForm = (adminPlayer) => {
  const objectives = world.scoreboard.getObjectives();
  const admins = objectives.filter((obj) =>
    obj.id.startsWith("sg_admin:admin_player:")
  );

  const form = new ActionFormData().title("Admin Players");

  if (admins.length === 0) {
    form.body("§7No players are currently admins.");
  } else {
    admins.forEach((admin) => {
      const playerName = admin.id.split(":")[2];
      const online = world.getPlayers().some((p) => p.name === playerName);
      form.button(
        `${playerName} - ${online ? "§aOnline" : "§cOffline"}`,
        `${online ? "textures/sg/admin/icons/admin.png" : "textures/sg/admin/icons/adminoffline.png"}`
      );
    });
  }

  if (adminPlayer.hasTag("sg_admin:owner")) {
    form.button("Add Admin", "textures/sg/admin/icons/add.png");
    if (admins.length > 0) {
      form.button("Remove Admin", "textures/sg/admin/icons/remove.png");
    }
  }

  form.button("§oGo Back");

  form.show(adminPlayer).then((res) => {
    if (res.canceled || res.selection === undefined) return;

    const goBackIndex = admins.length + (adminPlayer.hasTag("sg_admin:owner") ? 2 : 0);

    if (res.selection === goBackIndex) {
      showMainPage(adminPlayer); // Go back to the main page
    } else if (adminPlayer.hasTag("sg_admin:owner")) {
      if (res.selection === admins.length) {
        addAdminForm(adminPlayer); // Add Admin
      } else if (admins.length > 0 && res.selection === admins.length + 1) {
        removeAdminForm(adminPlayer); // Remove Admin
      } else if (res.selection < admins.length) {
        // Handle selection of an admin button, refresh the list
        viewAdminPlayersForm(adminPlayer); // Refresh the list
      }
    } else if (res.selection < admins.length) {
      // Handle selection of an admin button, refresh the list
      viewAdminPlayersForm(adminPlayer); // Refresh the list
    }
  });
};



const addAdminForm = (adminPlayer) => {
  const players = world.getPlayers();
  const form = new ModalFormData()
    .title("Add Admin")
    .dropdown("Select Player", players.map(p => p.name));

  form.show(adminPlayer).then(res => {
    if (res.canceled) return;

    const selectedPlayerIndex = res.formValues[0];
    const playerName = players[selectedPlayerIndex].name;
    const adminObjectiveName = `sg_admin:admin_player:${playerName}`;

    world.scoreboard.addObjective(adminObjectiveName, playerName);
    adminPlayer.sendMessage(`§aYou have added ${playerName} as an admin.`);
    players[selectedPlayerIndex].sendMessage(`§aYou are now an admin.`);
  });
};

const removeAdminForm = (adminPlayer) => {
  const objectives = world.scoreboard.getObjectives();
  const admins = objectives.filter(obj => obj.id.startsWith("sg_admin:admin_player:"));

  const form = new ModalFormData()
    .title("Remove Admin")
    .dropdown("Select Admin", admins.map(admin => admin.id.split(":")[2]));

  form.show(adminPlayer).then(res => {
    if (res.canceled) return;

    const selectedAdminIndex = res.formValues[0];
    const adminName = admins[selectedAdminIndex].id.split(":")[2];
    const adminObjectiveName = `sg_admin:admin_player:${adminName}`;
    adminPlayer.runCommand(`tellraw ${adminName} {"rawtext":[{ "text": "§cYou are no longer an admin"}]}`)
    system.run(()=>{
      world.scoreboard.removeObjective(adminObjectiveName);
    })
    adminPlayer.sendMessage(`§aYou have removed ${adminName} as an admin.`);
  });
};

world.afterEvents.playerSpawn.subscribe((event) => {
  const player = event.player;
  const banObjectiveName = `sg_admin:banned_player:${player.name}`;
  const bannedObjective = world.scoreboard.getObjective(banObjectiveName);

  if (bannedObjective) {
    world.sendMessage(`§c${player.name} is banned.`)
    event.player.runCommand(`kick ${player.name} §cYou are banned.`);
  } else {
    const adminObjectiveName = `sg_admin:admin_player:${player.name}`;
    const isAdmin = world.scoreboard.getObjective(adminObjectiveName);

    if (isAdmin) {
      world.sendMessage(`§c${player.name} is an admin.`);
    }
    if (player.hasTag("sg_admin:owner")) {
      world.sendMessage(`§c${player.name} is the owner.`);
    }
  }
});









world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  const playerObjectiveName = `sg_admin:owner_${player.name}`;
  const existingObjective = world.scoreboard.getObjective(playerObjectiveName);

  if (initialSpawn) {
    // If it's the player's first time joining with the add-on
    if (!existingObjective) {
      system.runTimeout(() => {
        // Assign the "owner" tag to the player
        if (!player.hasTag("sg_admin:owner")) {
          player.addTag("sg_admin:owner");
          player.sendMessage("§aYou have been assigned as the world owner for the Admin Mod!");

          // Add the player to a scoreboard objective to track that they have been designated as the owner
          world.scoreboard.addObjective(playerObjectiveName);
        }

        // Handle version check and guide delivery
        const version = player.getDynamicProperty("sg_admin_version");
        if (version === undefined) {
          player.setDynamicProperty("sg_admin_version", VERSION);
          if (player.hasTag("sg_admin_received_guide")) {
            player.removeTag("sg_admin_received_guide");
            player.sendMessage({
              translate: "sg_admin.announce_update",
              with: [VERSION],
            });
            return;
          }
          player.dimension.spawnItem(
            new ItemStack("sg_admin:guide"),
            player.location
          );
        } else if (version !== VERSION) {
          player.setDynamicProperty("sg_admin_version", VERSION);
          player.sendMessage({
            translate: "sg_admin.announce_update",
            with: [VERSION],
          });
        }
      }, 10 * TicksPerSecond);
    }
  }
});


const BUTTONS = [
  {
    // KICK PLAYER
    translation_key: "sg_admin.guide.kick_player.title",
    icon_path: "textures/sg/admin/icons/kick.png",
    action: (player) => {
      kickPlayerForm(player); // Opens the form to kick a player
    },
  },
  {
    // BAN PLAYER
    translation_key: "sg_admin.guide.ban_player.title",
    icon_path: "textures/sg/admin/icons/ban.png",
    action: (player) => {
      banPlayerForm(player); // Opens the form to ban a player
    },
  },
  {
    // VIEW BANNED PLAYERS
    translation_key: "sg_admin.guide.view_banned.title",
    icon_path: "textures/sg/admin/icons/banned.png",
    action: (player) => {
      viewBannedPlayersForm(player); // Opens the list of banned players
    },
  },
  {
    // VIEW ADMINS PLAYERS
    translation_key: "sg_admin.guide.view_admins.title",
    icon_path: "textures/sg/admin/icons/admins.png",
    action: (player) => {
      viewAdminPlayersForm(player); // Opens the list of banned players
    },
  },
  {
  // VANISH BUTTON
  translation_key: "sg_admin.guide.vanish.title",
  icon_path: "textures/sg/admin/icons/vanish.png",
    action: (player) => {
      // Broadcast fake leave message
      world.sendMessage(`§e${player.name} left the game`);
      
      // Store the player's current gamemode as a tag
      const currentGamemode = player.getGameMode();
      const gamemodeTag = `sg_admin:spectate:${currentGamemode}`;
      player.addTag(gamemodeTag);

      // Get the player's current coordinates and store as a tag
      const x = roundCoordinates(player.location.x);
      const y = roundCoordinates(player.location.y);
      const z = roundCoordinates(player.location.z);
      const positionTag = `sg_admin:spectate_position:${x},${y},${z}`;
      player.addTag(positionTag);
      
      // Change the player's gamemode to Spectator
      player.setGameMode(GameMode.spectator);
      
      // Notify the player that they have vanished
      player.sendMessage("§aYou are now in Spectator mode and have vanished from the game");
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_admin.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_admin.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_admin.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_admin.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_admin.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_admin.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_admin.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  // Check if the player has the "sg_admin:owner" tag or is listed as an admin
  if (player.hasTag("sg_admin:owner") || world.scoreboard.getObjectives().some(obj => obj.id === `sg_admin:admin_player:${player.name}`)) {
    showGuidePage(player, "main", 4, BUTTONS);
  } else {
    player.sendMessage("§cYou do not have permission to use the Admin Handbook.");
  }
};


world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_admin:guide") {
      showMainPage(event.source);
    }
  });
});